<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 */

/**
 * A class representing a user in PHPDevShell.
 *
 * Can be used to examine / modify an existing user, or create / import  a new user
 *
 * @date 20091125
 * @version 1.0.1
 * @author Greg, Jason
 */
class PHPDS_user extends user
{
	/**
	 * Stores string of groups for specific user.
	 *
	 * @var string
	 */
	public $mergeGroups;
	/**
	 * Stores string of roles for specific user.
	 *
	 * @var string
	 */
	public $mergeRoles;
	/**
	 * Set roles that exists.
	 *
	 * @var array
	 */
	public $rolesArray;
	/**
	 * Set groups that exists.
	 *
	 * @var array
	 */
	public $groupsArray;
	/**
	 * This function gets all role id's for a given user id, while returning a string divided by ',' character or an array with ids.
	 * To pull multiple user roles, provide a string for $user_ids like so: '2,5,10,19'.
	 *
	 * @param string $user_id
	 * @param boolean $return_array
	 * @return mixed If $return_array = false a comma delimited string will be returned, else an array.
	 * @author Jason Schoeman <titan@phpdevshell.org>
	 */
	public function getRoles($user_id = false, $return_array = false)
	{
		return $this->db->invokeQuery('USER_getRolesQuery', $user_id, $return_array);
	}

	/**
	 * This function gets all group id's for given user ids, while returning a string divided by ',' character or an array with ids.
	 * To pull multiple user groups, provide a string for $user_ids like so : '2,5,10,19'.
	 *
	 * @param string $user_id Leave this field empty if you want skip if user is root.
	 * @param boolean $return_array
	 * @return mixed If $return_array = false a comma delimited string will be returned, else an array.
	 * @author Jason Schoeman <titan@phpdevshell.org>
	 */
	public function getGroups($user_id = false, $return_array = false, $alias_only = false)
	{
		// $alias_only is deprecated and replaced by tagger.
		$groups = $this->db->invokeQuery('USER_getGroupsQuery', $user_id, $return_array);
		return $groups;
	}

	/**
	 * Simple check to see if a certain role exists.
	 *
	 * @param integer $role_id
	 * @return boolean
	 */
	public function roleExist($role_id)
	{
		return $this->db->invokeQuery('USER_roleExistQuery', $role_id);
	}

	/**
	 * Simple check to see if a certain group exists.
	 *
	 * @param integer $group_id
	 * @return boolean
	 */
	public function groupExist($group_id)
	{
		return $this->db->invokeQuery('USER_groupExistQuery', $group_id);
	}

	/**
	 * Check if user belongs to given role. Returns true if user belongs to user role.
	 *
	 * @param integer $user_id
	 * @param integer $user_role
	 * @return boolean Returns true if user belongs to user role.
	 * @author Jason Schoeman <titan@phpdevshell.org>
	 */
	public function belongsToRole($user_id = false, $user_role)
	{
		return $this->db->invokeQuery('USER_belongsToRoleQuery', $user_id, $user_role);
	}

	/**
	 * Check if user belongs to given group. Returns true if user belongs to user group.
	 *
	 * @param integer $user_id
	 * @param integer $user_group
	 * @return boolean Returns true if user belongs to user group.
	 * @author Jason Schoeman <titan@phpdevshell.org>
	 */
	public function belongsToGroup($user_id = false, $user_group)
	{
		// We need to check what user group this user belongs to.
		// This user belongs to root group, so it is safe to allow.
		if ($this->user->isRoot($user_id)) return true;
		// Check if we have a user id.
		if (empty($user_id)) {
			// Use default id.
			(!empty($this->configuration['user_id'])) ? $user_id = $this->configuration['user_id'] : $user_id = false;
		}
		// Call groups and list them all.
		$user_groups_arr = array_flip($this->getGroups($user_id, true));
		// Lets check if this user exists.
		if (array_key_exists($user_group, $user_groups_arr)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Creates a query to extend a role query, it will return false if user is root so everything can get listed.
	 * This is meant to be used inside an existing role query.
	 *
	 * @param string $query_request Normal query to be returned if user is not a root user.
	 * @param string $query_root_request If you want a query to be processed for a root user seperately.
	 * @return mixed
	 */
	public function setRoleQuery($query_request, $query_root_request = false)
	{
		// Check if it is a root user, if it is, we dont need to return anything cause everything should be listed.
		if ($this->user->isRoot()) {
			// This user belongs to root role, so it is safe to allow.
			if (!empty($query_root_request)) {
				return " $query_root_request ";
			} else {
				return false;
			}
		} else {
			// Set query line.
			return " $query_request ";
		}
	}

	/**
	 * Creates a query to extend a group query, it will return false if user is root so everything can get listed.
	 * This is meant to be used inside an existing group query.
	 *
	 * @param string $query_request Normal query to be returned if user is not a root user.
	 * @param string $query_root_request If you want a query to be processed for a root user seperately.
	 * @return mixed
	 */
	public function setGroupQuery($query_request, $query_root_request = false)
	{
		// Check if it is a root user, if it is, we dont need to return anything cause everything should be listed.
		if ($this->user->isRoot()) {
			// This user belongs to root group, so it is safe to allow.
			if (!empty($query_root_request)) {
				return " $query_root_request ";
			} else {
				return false;
			}
		} else {
			// Set query line.
			return " $query_request ";
		}
	}

	/**
	 * Search the database for the given credentials
	 *
	 * @param string $username
	 * @param string $password
	 * @return array or false the user record
	 * @date 20100204 (v1.0) (greg) created from Jason's original fct
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @version 1.0.1
	 * @author jason, greg
	 */
	public function lookupUser($username, $password = '')
	{
		$password = $this->security->hashPassword($password);
		return $this->db->invokeQuery('USER_selectUserQuery', $username, $password);
	}

	/**
	 * Search the database for the given credentials from a persistent cookie
	 *
	 * @param string $cookie
	 * @return array or false the user record
	 * @date 20100702 (v1.0.0) (ross) created from the lookup_user fct by jason and greg
	 * @version 1.0.0
	 * @author jason, greg, ross
	 */
	public function lookupPersistentUser($cookie)
	{
		return $this->db->invokeQuery('USER_selectUserPersistentQuery', $cookie);
	}

	/**
	 * Set a persistent cookie to be used as a remember me function
	 *
	 * @param string $cookie
	 * @return array or false the user record
	 * @date 20100702 (v1.0.0) (ross) created function
	 * @version	1.0.0
	 * @author ross
	 */
	public function setPersistentCookie($user_id)
	{
		return $this->db->invokeQuery('USER_setPersistentCookieQuery', $user_id);
	}

	/**
	 * Delete the current persistent cookie from the db and kill the cookie on the user end.
	 *
	 * @return boolean
	 * @date 20100702 (v1.0.0) (ross) created function
	 * @version 1.0.0
	 * @author ross
	 */
	public function deletePersistentCookie($user_id)
	{
		return $this->db->invokeQuery('USER_deletePersistentCookieQuery', $user_id);
	}

	/**
	 * Sets all settings to guest account.
	 *
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @return string
	 * @author Jason Schoeman, greg
	 */
	public function setGuest()
	{
		// Check if we need to set the guest.
		if (!isset($_SESSION['user_id'])) {
			// Get required settings.
			$settings_array = $this->db->essentialSettings;
			// Get user_role_name and user_group_name.
			$user_group_name = $this->db->invokeQuery('USER_guestGroupNameQuery');
			$user_role_name = $this->db->invokeQuery('USER_guestRoleNameQuery');

			// Check if we have a timezone to display.
			if (!empty($this->configuration['system_timezone'])) {
				// Assign server timezone.
				$user_timezone = $this->configuration['system_timezone'];
			} else {
				// Try and locate correct timezone.
				$user_timezone = date_default_timezone_get();
			}
			// Set guest settings.
			$_SESSION['user_name'] = ___('Guest User');
			$_SESSION['user_display_name'] = ___('Guest User');
			$_SESSION['user_role_name'] = ___("$user_role_name");
			$_SESSION['user_group_name'] = ___("$user_group_name");
			$_SESSION['user_group'] = $settings_array['guest_group'];
			$_SESSION['user_role'] = $settings_array['guest_role'];
			$_SESSION['user_email'] = false;
			$_SESSION['user_language'] = $this->configuration['language'];
			$_SESSION['user_region'] = $this->configuration['region'];
			$_SESSION['user_timezone'] = $user_timezone;
			$_SESSION['user_locale'] = $this->core->formatLocale();
			$_SESSION['user_id'] = 0;
		}
	}

	/**
	 * Destroys login session data.
	 *
	 * @author Jason Schoeman
	 */
	public function logout($set_guest = true)
	{
		$this->deletePersistentCookie($_SESSION['user_id']);
		// Log this login.
		$this->db->logArray[] = array('log_type' => 5, 'log_description' => ___('Logged-out'));
		// Unset some Session variables.
		unset($_SESSION['user_email']);
		unset($_SESSION['user_id']);
		unset($_SESSION['user_name']);
		unset($_SESSION['user_group']);
		unset($_SESSION['user_role']);
		unset($_SESSION['user_display_name']);
		unset($_SESSION['user_group_name']);
		unset($_SESSION['user_role_name']);
		unset($_SESSION['user_language']);
		unset($_SESSION['user_timezone']);
		unset($_SESSION['user_region']);
		unset($_SESSION['user_locale']);
		// Clear cache.
		$this->db->cacheClear();
		// Unset all of the session variables.
		$_SESSION = array();
		// Destroy the session.
		session_destroy();
		// Set guest if required.
		if ($set_guest) {
			// Set guest.
			$this->setGuest();
		}
	}

	/**
	 * Check if user is a root user.
	 *
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @param mixed $user_id If not logged in user, what user should be checked (primary role check only).
	 * @return boolean
	 */
	public function isRoot($user_id = false)
	{
		// Check if we are doing a custom check on a user.
		// If we have a given user id we need to handle this differently.
		if (!empty($user_id)) {
			// Check if we are checking an active logged in user.
			if ($this->configuration['user_id'] == $user_id) {
				// We are checking a logged in user.
				if ($this->configuration['user_role'] == $this->configuration['root_role']) {
					// Is root! Nice!
					return true;
				} else {
					// We did not get anywhere, this user is not a root user.
					return false;
				}
			} else {
				// Oh ok, we need to delve deeper so we can find out who this user is.
				$check_role_id = $this->db->invokeQuery('USER_isRootQuery', $user_id);
				// Ok is this a role user or not.
				if ($check_role_id == $this->configuration['root_role']) {
					return true;
				} else {
					return false;
				}
			}
		} else if (($this->configuration['user_role'] == $this->configuration['root_role'])) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Returns current logged in user id.
	 *
	 * @return integer
	 */
	public function currentUserID()
	{
		return $this->configuration['user_id'];
	}

	/**
	 * Check if the currently logged in user is the same group as the given user.
	 *
	 * This can be used to check if the current user is allowed access to the given user's data
	 *
	 * @date 20100222
	 * @version	1.0
	 * @author greg
	 * @param $user_id integer, the ID of the other user
	 * @return boolean, whether the current user is in the same group
	 * @see	http://wiki.phpdevshell.org/wiki/Security_with_groups_and_roles
	 */
	public function isSameGroup($user_id)
	{
		$edit = $this->db->invokeQuery('USER_isSameGroupQuery', $user_id);
		return (!empty($edit['user_id']));
	}

	/**
	 * Checks to see if user and password is correct and allowed. Then creates session data accordingly.
	 *
	 * @param string $username
	 * @param string $password
	 * @return boolean
	 * @date 20100204 greg: split into pieces
	 * @version	1.1
	 * @author jason, greg
	 */
	public function login($username, $password)
	{
		$user_array = $this->lookupUser($username, $password);
		if (!empty($user_array)) {
			$this->doLogin($user_array);
			if ($this->db->essentialSettings['allow_remember'] && isset($this->security->post['user_remember'])) {
				$this->setPersistentCookie($user_array['user_id']);
			}
			$this->info(sprintf(___('Successful login for "%s" to user id %s'), $username, $_SESSION['user_id']));
			return true;
		} else {
			$this->info(sprintf(___('Rejected login for "%s"'), $username));
			return false;
		}
	}

	/**
	 * Simple method to return users IP, this method will be improved in the future if needed.
	 *
	 * @return string
	 */
	public function userIp()
	{
		return $this->getUserIp();
	}

	/**
	 * Simple method to return users IP, this method will be improved in the future if needed.
	 *
	 * @version 1.0.1
	 * @date 20110315 (v1.0.1) (greg) fix a possible undef when not used through a webserver
	 *
	 * @return string
	 */
	public function getUserIp()
	{
		// Extremely basic form of IP tracking.
		if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			// If user comes from proxy.
			return $_SERVER['HTTP_X_FORWARDED_FOR'];
		} else {
			// No proxy, show remote address.
			return isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : false;
		}
	}

	/**
	 * Check is user is logged in, return false if not.
	 *
	 * @return boolean
	 */
	public function isLoggedIn()
	{
		// Do a simple check to see if user is logged in.
		if (empty($this->configuration['user_id'])) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Check if a user has access to a given menu id.
	 *
	 * @version 1.0.1
	 * @date 20091105 fixed a possible warning when the menu is not in the list (i.e. the user is not allowed)
	 *
	 * @param mixed This can have both the menu id as an integer or as a string.
	 * @param string The type of item requested, menu_id, menu_name etc...
	 * @return boolean Will return requested variable if user has access to requested menu item menu item.
	 */
	public function canAccessMenu($menu_id, $type = 'menu_id')
	{
		// Do a simple check to see if menu id exists.
		if (!empty($this->navigation->navigation[$menu_id][$type])) {
			// If access is allowed return true.
			return $this->navigation->navigation[$menu_id][$type];
		} else {
			// If access is denied return false.
			return false;
		}
	}

	/**
	 * Make the given user the logged in user
	 *
	 * @param array $select_user_array
	 * @return nothing
	 * @date 20100204 greg: created from Jason's original fct
	 * @version	1.0
	 * @author jason, greg
	 */
	public function doLogin($select_user_array, $persistent = false)
	{
		$user_name_db = $select_user_array['user_name'];
		$user_user_password_db = $select_user_array['user_password'];
		$user_display_name_db = $select_user_array['user_display_name'];
		$user_email_db = $select_user_array['user_email'];
		$user_id_db = $select_user_array['user_id'];
		$user_group_db = $select_user_array['user_group'];
		$user_role_db = $select_user_array['user_role'];
		$user_group_name_db = $select_user_array['user_group_name'];
		$user_role_name_db = $select_user_array['user_role_name'];
		$user_language_db = $select_user_array['language'];
		$user_region_db = $select_user_array['region'];
		// Check if we have a user timezone.
		if (!empty($select_user_array['user_timezone'])) {
			// If we do, assign it.
			$user_timezone_db = $select_user_array['user_timezone'];
		} else if (!empty($this->configuration['system_timezone'])) {
			// If we dont, user servers timezone.
			$user_timezone_db = $this->configuration['system_timezone'];
		} else {
			// Try and locate correct timezone.
			$user_timezone_db = date_default_timezone_get();
		}
		//if ((! empty($user_display_name_db)) && (! empty($user_id_db)) && (($username === $user_name_db) || ($username === $user_email_db)) && ($password === $user_user_password_db)) {
		$_SESSION['user_display_name'] = $user_display_name_db;
		$_SESSION['user_email'] = $user_email_db;
		$_SESSION['user_id'] = $user_id_db;
		$_SESSION['user_name'] = $user_name_db;
		$_SESSION['user_group'] = $user_group_db;
		$_SESSION['user_role'] = $user_role_db;
		$_SESSION['user_role_name'] = $user_role_name_db;
		$_SESSION['user_group_name'] = $user_group_name_db;
		$_SESSION['user_timezone'] = $user_timezone_db;
		// Check if language is empty.
		if (!empty($user_language_db)) {
			$user_language = $user_language_db;
		} else {
			$user_language = $this->configuration['language'];
		}
		// Check if region is empty.
		if (!empty($user_region_db)) {
			$user_region = $user_region_db;
		} else {
			$user_region = $this->configuration['region'];
		}
		// Assign region and language.
		$_SESSION['user_language'] = $user_language;
		$_SESSION['user_region'] = $user_region;
		// Set user_locale.
		$_SESSION['user_locale'] = $this->core->formatLocale(true, $user_language, $user_region);
		// Log this login.
		if (!$persistent) {
			$this->db->logArray[] = array('log_type' => 4, 'user_id' => $user_id_db, 'logged_by' => $user_display_name_db, 'log_description' => ___('Logged-in'));
		} else {
			$this->db->logArray[] = array('log_type' => 4, 'user_id' => $user_id_db, 'logged_by' => $user_display_name_db, 'log_description' => $persistent);
		}

		// Clear cache.
		$this->db->cacheClear();
	}
}
